/*****************************************************************************

NAME:
   main.c -- detect spam and bogons presented on standard input.

AUTHOR:
   Eric S. Raymond <esr@thyrsus.com>

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <limits.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "bogofilter.h"

#define BOGODIR		"/.bogofilter/"
#define GOODFILE	BOGODIR "goodlist"
#define BADFILE		BOGODIR "badlist"
#define FIFOFILE	BOGODIR "fifo"
#define PIDFILE		BOGODIR "pid"

int verbose;

int main(int argc, char **argv)
{
    int	ch, client_mode = 0;
    int register_spam = 0, register_nonspam = 0;
    char	goodfile[PATH_MAX], badfile[PATH_MAX];
    char	fifofile[PATH_MAX], pidfile[PATH_MAX];
    struct stat sb;
    int readerror=0;

    strcpy(goodfile, getenv("HOME"));
    strcat(goodfile, BOGODIR);
    ch = stat(goodfile, &sb);
    if (!((ch == 0 && S_ISDIR(sb.st_mode))
	  || (errno==ENOENT && mkdir(goodfile, S_IRUSR|S_IWUSR|S_IXUSR)==0)))
    {
	fprintf(stderr,"bogofilter: something is wrong with %s.\n", goodfile);
	exit(2);
    }

    strcpy(goodfile, getenv("HOME"));
    strcat(goodfile, GOODFILE);
    strcpy(badfile, getenv("HOME"));
    strcat(badfile, BADFILE);
    strcpy(fifofile, getenv("HOME"));
    strcat(fifofile, FIFOFILE);
    strcpy(pidfile, getenv("HOME"));
    strcat(pidfile, PIDFILE);

    while ((ch = getopt(argc, argv, "snvc")) != EOF)
	switch(ch)
	{
	case 's':
	    register_spam = 1;
	    break;

	case 'n':
	    register_nonspam = 1;
	    break;

	case 'c':
	    client_mode = 1;
	    break;

	case 'v':
	    verbose++;
	    break;
	}

    readerror += read_list(goodfile, &good_list);
    readerror += read_list(badfile,  &bad_list);

    /* readerror is ok, but only if using register_* modes */
    if (readerror && ! (register_spam || register_nonspam)) {
	fprintf(stderr, "Error: can't open list file(s).\n");
	exit(2);
    }

    if (client_mode)	// start client, fork server
    {
	if (register_spam)
	    ;
	else if (register_nonspam)
	    ;
	else
	    exit(0);
    }
    else
    {
	if (register_spam)
	{
	    register_words(0, &bad_list);
	    write_list(&bad_list);
	    if (verbose)
		printf("bogofilter: %lu messages on the bad list\n", bad_list.msgcount);
	}
	else if (register_nonspam)
	{
	    register_words(0, &good_list);
	    write_list(&good_list);
	    if (verbose)
		printf("bogofilter: %lu messages on the good list\n", good_list.msgcount);
	}
	else
	    exit(bogofilter(0));
    }

    exit(0);
}

// End
